################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

## font.tcl Procedures for changing fonts
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

## The procedures in this file are responsible for font display

## Conventions:
##   All procedures in this file begin with tkW3Font

# phxW3FontInitialize creates an array phxW3Font of fonts for
# which we have at least 6 sizes.  These fonts are installed into the
# Font menu.

proc phxW3FontInitialize {} {
    global phxW3FontBin phxW3Fonts
    catch {unset phxW3FontBin}
    set menu .menu.font.m

    set maskFB {^(-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*)--([^-]*-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-.*)$}
    set maskFront {^-([^-]*)-([^-]*)-([^-]*)-([^-]*)-([^-]*)$}
    set maskBack {^([^-]*)-([^-]*)-([^-]*)-([^-]*)-([^-]*)-([^-]*)-(.*)$}
    set maskWidth {^-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*--[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-([^-]*)-.*$}

    # Get all fonts.  Place roman fonts with nonzero width into bins.
    set phxW3Fonts [split [string tolower [string trim [exec xlsfonts ?*-*-*-*-*--*-*-*-*-*-*-*-*]]] \n]
    foreach font $phxW3Fonts {
	regexp $maskFB $font all front back
	regexp $maskFront $front all foundry family weight slant setwidth
	regexp $maskBack $back all pixels points xres yres spacing width charset
	if {$slant == "r" && $width != 0} {
	    lappend phxW3FontBin(-*-$family-*-r-$setwidth--*-*-$xres-$yres-*-*-$charset) [list $family $xres $width $font]
	}
    }

    # Go through the bins
    foreach choice [lsort -command "phxW3FontCompareFamilyXres" \
	[array names phxW3FontBin]] \
    {
	# Sort by width.
	set phxW3FontBin($choice) \
	    [lsort -command "phxW3FontCompareWidth" $phxW3FontBin($choice)]

	# Eliminate duplicate width values.
	set length [llength $phxW3FontBin($choice)]
	for {set i 0} {$i + 1 < $length} {} {
	    set fontOne [lindex [lindex $phxW3FontBin($choice) $i] 3]
	    set fontTwo [lindex [lindex $phxW3FontBin($choice) [incr i]] 3]
	    regexp $maskWidth $fontOne all widthOne
	    regexp $maskWidth $fontTwo all widthTwo
	    if {$widthOne == $widthTwo} {
		set phxW3FontBin($choice) [lreplace $phxW3FontBin($choice) $i $i]
		incr i -1
		incr length -1
	    }
	}

	# We prefer at least 7 sizes but will accept 6 or more.
	set length [llength $phxW3FontBin($choice)]
	if {$length < 6} { unset phxW3FontBin($choice) ; continue }

	for {set i 0} {$i + 5 < $length} {incr i} {
	    regexp $maskFB [lindex [lindex $phxW3FontBin($choice) $i] 3] \
		all front back
	    regexp $maskFront $front all foundry family weight slant setwidth
	    regexp $maskBack $back \
		all pixels points xres yres spacing width charset
	    if {$weight == "medium"} {
		set submenu $menu.[join $family "-"]
		if {![winfo exists $submenu]} {
		    PhxApp_standardize_menuwidg [menu $submenu]
		    $menu add cascade -label "$family" -menu $submenu
		}
		$submenu add command \
		    -command [list phxW3FontNew $choice $i] \
		    -label "[expr {$points/10}]pt (at resolution = $xres)"
	    }
	}
    }

    # Choose an initial font.  Make it the second largest available.
    foreach try {times new-century-schoolbook helvetica palatino} {
	if {[winfo exists $menu.$try]} {
	    set index [expr {[$menu.$try index last] - 1}]
	    if {$index < 0} {set index 0}
	    $menu.$try invoke $index
	    break
	}
    }
}

# Compare two array entries of phxW3Font.  Sort on $family first,
# $xres second, then the entire font name of the smallest font in
# this group.

proc phxW3FontCompareFamilyXres {a b} {
    global phxW3FontBin
    set a [lindex $phxW3FontBin($a) 0]
    set b [lindex $phxW3FontBin($b) 0]
    if {[lindex $a 0] < [lindex $b 0]} { return -1 }
    if {[lindex $a 0] > [lindex $b 0]} { return +1 }
    if {[lindex $a 1] < [lindex $b 1]} { return -1 }
    if {[lindex $a 1] > [lindex $b 1]} { return +1 }
    if {[lindex $a 3] < [lindex $b 3]} { return -1 }
    if {[lindex $a 3] > [lindex $b 3]} { return +1 }
    return 0
}

# Compare the widths of two fonts.

proc phxW3FontCompareWidth {a b} {
    return [expr {[lindex $a 2] - [lindex $b 2]}]
}

# phxW3FontNew: Change the Displayed Font Family.

proc phxW3FontNew {font index} {
    global phxW3FontBin phxW3Fonts
    set maskFB {^(-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*)--([^-]*-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-.*)$}
    set maskFront {^-([^-]*)-([^-]*)-([^-]*)-([^-]*)-([^-]*)$}
    set maskBack {^([^-]*)-([^-]*)-([^-]*)-([^-]*)-([^-]*)-([^-]*)-(.*)$}

    set font [lrange $phxW3FontBin($font) $index [expr {$index + 6}]]
    set baseFont [lindex [lindex $font 0] 3]
    regexp $maskFB $baseFont all front back
    regexp $maskFront $front all foundry family weight slant setwidth
    regexp $maskBack $back all pixels points xres yres spacing width charset

    set fontStyles ""
    lappend fontStyles [list "" $baseFont]

    # Use the next 6 bigger fonts for H6, ..., H1.  If there are only
    # five bigger fonts then make H6 the same as plain text.
    set biggest [llength $font]
    for {set i 1} {$i <= 6} {incr i} {
	lappend fontStyles [list "H$i" [lindex [lindex $font [expr {$biggest-$i}]] 3]]
    }

    set Bold $baseFont
    foreach try {bold-r} {
	set attempt "-*-$family-$try-$setwidth--$pixels-$points-$xres-$yres-*-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set Bold [lindex $phxW3Fonts $index]
	    break
	}
    }

    set Italic $baseFont
    foreach try {medium-i medium-o} {
	set attempt "-*-$family-$try-$setwidth--$pixels-$points-$xres-$yres-*-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set Italic [lindex $phxW3Fonts $index]
	    break
	}
    }

    set Teletype $baseFont
    set teletypeWidth $pixels
    foreach try {0 1 -1 2 -2 3 -3 4 -4} {
	incr try $pixels
	set attempt "-*-courier-medium-r-normal--$try-*-*-*-m-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set Teletype [lindex $phxW3Fonts $index]
	    set teletypeWidth $try
	    break
	}
    }

    set smallTeletype $Teletype
    foreach try {-1 -2 -3} {
	incr try $teletypeWidth
	set attempt "-*-courier-medium-r-normal--$try-*-*-*-m-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set smallTeletype [lindex $phxW3Fonts $index]
	    break
	}
    }

    set BoldItalic $Italic
    foreach try {bold-i bold-o bold-r} {
	set attempt "-*-$family-$try-$setwidth--$pixels-$points-$xres-$yres-*-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set BoldItalic [lindex $phxW3Fonts $index]
	    break
	}
    }

    set BoldTeletype $Teletype
    foreach try {bold-r} {
	set attempt "-*-courier-$try-normal--$teletypeWidth-*-*-*-m-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set BoldTeletype [lindex $phxW3Fonts $index]
	    break
	}
    }

    set ItalicTeletype $Teletype
    foreach try {medium-i medium-o} {
	set attempt "-*-courier-$try-normal--$teletypeWidth-*-*-*-m-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set ItalicTeletype [lindex $phxW3Fonts $index]
	    break
	}
    }

    set BoldItalicTeletype $ItalicTeletype
    foreach try {bold-i bold-o bold-r} {
	set attempt "-*-courier-$try-normal--$teletypeWidth-*-*-*-m-*-$charset"
	if {[set index [lsearch -glob $phxW3Fonts $attempt]] >= 0} {
	    set BoldItalicTeletype [lindex $phxW3Fonts $index]
	    break
	}
    }

# The CharStyles and ParaStyles exist in three places: (Merge, if possible!!)
#    phxW3FontNew
#    config.tcl
#    phxW3EditConvertToHTML
    lappend fontStyles [list "ADDRESS" $Teletype]
    lappend fontStyles [list "B" $Bold]
    # No need to declare BLOCKQUOTE since it doesn't change the font.
    #lappend fontStyles [list "BLOCKQUOTE" $baseFont]
    lappend fontStyles [list "CITE" $Italic]
    lappend fontStyles [list "CODE" $Teletype]
    lappend fontStyles [list "DFN" $Bold]
    lappend fontStyles [list "EM" $Italic]
    lappend fontStyles [list "I" $Italic]
    lappend fontStyles [list "KBD" $BoldTeletype]
    lappend fontStyles [list "PRE" $smallTeletype]
    lappend fontStyles [list "SAMP" $Teletype]
# STRIKE is text with strike-out.  (Not supported)
    lappend fontStyles [list "STRONG" $Bold]
    lappend fontStyles [list "TT" $Teletype]
# U is underlining.  (Not supported by a font change.)
    lappend fontStyles [list "VAR" $BoldItalic]
    lappend fontStyles [list "XMP" $smallTeletype]

    # Update global variables
    global phxW3FontCurrent
    set phxW3FontCurrent $fontStyles

    global phxW3FontHeight
    catch {text .fontTest -relief flat -bg gray -height 10}
    .fontTest configure -font $baseFont
    pack configure .fontTest -before .f -side bottom
    update
    set phxW3FontHeight [expr {[winfo height .fontTest]/10}]
    pack forget .fontTest

    # Change to the desired font.
    tkW3FontSetTextFonts {} $fontStyles

    # Hack to make sure .titles.title_entry resizes itself!!
    tkW3OutputSetAddress [tkW3OutputGetAddress] [tkW3OutputGetTitle]
}

proc phxW3FontGetHeight {} {
    global phxW3FontHeight
    if {[info exists phxW3FontHeight]} {return $phxW3FontHeight}
    return ""
}

proc tkW3FontSetTextFonts {textboxes {list ""}} {
    global phxW3FontCurrent

    tkW3OutputCursorWait

    if {$textboxes == ""} {
	set windows [PhxUtil_get_descendants {.f .titles}]
	foreach w $windows {
	    if {[winfo class $w] == "Text"} {
		lappend textboxes $w
	    }
	}
    }

    if {$list == ""} { set list $phxW3FontCurrent }

    foreach w $textboxes {
	foreach font $list {
	    set style [lindex $font 0]
	    if {$style == ""} {
		$w configure -font [lindex $font 1]
	    } else {
		$w tag configure $style -font [lindex $font 1]
	    }
	}
	$w tag configure U -underline true
    }
    update
    tkW3OutputCursorNormal
}
